
#ifndef SYSTEMSETTINGSGLOBAL_H
#define SYSTEMSETTINGSGLOBAL_H

#include <QtGlobal>

#if defined(SYSTEMSETTINGS_BUILD_LIBRARY)
#define SYSTEMSETTINGS_EXPORT Q_DECL_EXPORT
#else
#define SYSTEMSETTINGS_EXPORT Q_DECL_IMPORT
#endif

#endif
