/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -p mceiface.h:mceiface.cpp mce.xml
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef MCEIFACE_H
#define MCEIFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface com.nokia.mce.request
 */
class ComNokiaMceRequestInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "com.nokia.mce.request"; }

public:
    ComNokiaMceRequestInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~ComNokiaMceRequestInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QDBusVariant> get_config(const QDBusObjectPath &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("get_config"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> get_config_all()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("get_config_all"), argumentList);
    }

    inline QDBusPendingReply<uint> get_radio_states()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("get_radio_states"), argumentList);
    }

    inline QDBusPendingReply<> req_radio_states_change(uint states, uint mask)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(states) << QVariant::fromValue(mask);
        return asyncCallWithArgumentList(QStringLiteral("req_radio_states_change"), argumentList);
    }

    inline QDBusPendingReply<bool> set_config(const QDBusObjectPath &path, const QDBusVariant &value)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path) << QVariant::fromValue(value);
        return asyncCallWithArgumentList(QStringLiteral("set_config"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

/*
 * Proxy class for interface com.nokia.mce.signal
 */
class ComNokiaMceSignalInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "com.nokia.mce.signal"; }

public:
    ComNokiaMceSignalInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~ComNokiaMceSignalInterface();

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void config_change_ind(const QString &key, const QDBusVariant &value);
    void display_status_ind(const QString &state);
};

namespace com {
  namespace nokia {
    namespace mce {
      typedef ::ComNokiaMceRequestInterface request;
      typedef ::ComNokiaMceSignalInterface signal;
    }
  }
}
#endif
